#ctc_player
#The player class holds all information about a player's progression and status.

import ctc_unit

class Player:
    def __init__(self, color, number, points=150):
        self.color = color
        self.number = number
        self.points = points
        self.points_spent = 0

        self.castle_Location = None
        self.castle_Placed = False
        #player places their castle along with their army

        self.battles_Won = 0
        self.battles_Fought = 0

        self.ALL_UNITS = []
        self.units_Trimmed = False
        self.MAX_ARMY_SIZE = 10
        for i in range(0, self.MAX_ARMY_SIZE):
            self.ALL_UNITS.append(None)

        self.MOVES = 3
    def set_Castle_Location(self, location):
        castle_X = location[0]
        castle_Y = location[1]

        self.castle_Location = (castle_X, castle_Y)
    def add_Unit(self, unit_Index, unit_Type):
        """Governs addition of units to the army and soldiers to the units."""
        
        theUnit = self.ALL_UNITS[unit_Index]

        if unit_Type.cost > self.points:
            return

        if theUnit == None: #no unit in this slot, add one
            self.ALL_UNITS[unit_Index] = ctc_unit.Unit(unit_Type, self.color)
            self.points -= unit_Type.cost
            self.points_spent += unit_Type.cost
            return
        elif theUnit.unit_Type.type_Name == unit_Type.type_Name:
            #existing type matches add type
            if theUnit.soldiers == 10: #can't have more than 10 soldiers
                return
            else:
                theUnit.soldiers += 1
                self.points -= unit_Type.cost
                self.points_spent += unit_Type.cost
            return
        else: #existing type doesn't match
            for index, unit in enumerate(self.ALL_UNITS):
                if unit == None: #find new slot and add
                    self.ALL_UNITS[index] = ctc_unit.Unit(unit_Type, self.color)
                    self.points -= unit_Type.cost
                    self.points_spent += unit_Type.cost
                    return index
            else: #else report failure
                return "Could not Add a Soldier of that Type"
                    
    def remove_Unit(self, unit_Index):
        """Removes soldiers from a unit, or the entire unit if last soldier."""
        theUnit = self.ALL_UNITS[unit_Index]

        if theUnit == None: #no unit here, do nothing
            return
        elif theUnit.soldiers == 1: #remove a soldier
            self.points += theUnit.unit_Type.cost
            self.points_spent -= theUnit.unit_Type.cost
            self.ALL_UNITS[unit_Index] = None
            return
        else: #only one soldier: remove the unit
            self.points += theUnit.unit_Type.cost
            self.points_spent -= theUnit.unit_Type.cost
            theUnit.soldiers -= 1
            return
        
    def add_Soldier_to_Unit(self, unit_Index, unit_Type):
        """Adds a new soldier directly to an existing unit.

        This function to be used only be a slot-specific "plus" button,
        which may not be included."""
        
        if unit_Index >= self.MAX_ARMY_SIZE:
            return "Unit Index Out of Bounds"
        elif self.ALL_UNITS[unit_Index] == None:
            self.ALL_UNITS[unit_Index].soldiers += 1
    def all_Units_Placed(self):
        """return False if any unit has location==None. True otherwise."""
        if not self.units_Trimmed:
            #should raise an error
            pass

        for unit in self.ALL_UNITS:
            if unit.location == None:
                return False

        return True
        
    def trim_Unit_List(self):
        """Removes empty slots (unit==None) from self.ALL_UNITS.

        This function must be called only after the build army phase
        is complete."""
        clean_Unit_List = []
        for unit in self.ALL_UNITS:
            if unit == None or unit.soldiers < 1:
                pass
            else:
                clean_Unit_List.append(unit)

        self.units_Trimmed = True
        self.ALL_UNITS = clean_Unit_List
    def next_Unplaced_Unit(self):
        """Returns the next unit where location==None; False if none exists."""
        if not self.units_Trimmed:
            #should raise an error
            pass
        
        for unit in self.ALL_UNITS:
            if unit.location == None:
                return unit
        return False
    def reset_Units(self):
        """Restores units to max moves and removes units with no soldiers."""

        
        for unit in self.ALL_UNITS:
            if unit.soldiers == 0:
                unit = None
            else:
                unit.current_Move = unit.unit_Type.move
                unit.valid_Moves = None

        self.trim_Unit_List()

    def print_Army(self):
        """Debug method to print out the current contents of the army."""
        index = 1
        for unit in self.ALL_UNITS:
            print "Unit ", str(index) + ":", unit.soldiers, unit.unit_Type.type_Name

    def clear_Army(self):
        for index, unit in enumerate(self.ALL_UNITS):
            if not unit == None:
                point_Total = unit.soldiers * unit.unit_Type.cost
                self.points += point_Total
                self.ALL_UNITS[index] = None

    def army_Not_Empty(self):
        """Returns False is all player units == None, True otherwise."""
        for unit in self.ALL_UNITS:
            if not unit == None:
                return True

        return False
