#ctc_screen
#This class and its derived classes are self-assembling "screens" -
#disrete sections of player view, such as the intro screen, army building
#screen, and map screen.

import pygame, pygbutton, sys, copy
from pygame.locals import *

import ctc_terrain
from ctc_imports import image_Path

BLACK     = (  0,   0,   0)
WHITE     = (255, 255, 255)
DARKGRAY  = ( 64,  64,  64)
GRAY      = (115, 115, 115)
LIGHTGRAY = (212, 208, 200)

#image_Path = "C:\\Python27\\ctc\\images\\"

class Screen:
    """Virtual base class for screen functionality."""
    def __init__(self, GameObject):
        self.BUTTONS = []

        self.screen_Image = None
        #self.image holds the pygame Surface with the screen graphic on it
    def handle_Event(self, event):
        for button in self.BUTTONS:
            button.handle_Input(event)
    def render(self):
        """Calls on the Renderer class to draw the screen's current state."""
        pass

class intro_Screen(Screen):
    def __init__(self):
        self.BUTTONS = []
        self.create_Buttons()
    def create_Buttons(self):
        playButtonRect = (650, 250, 125, 25)
        playButton = pygbutton.PygButton(playButtonRect, "Fight!")
        playButton.name = "playButton"
        self.BUTTONS.append(playButton)

        learnButtonRect = (650, 300, 125, 25)
        learnButton = pygbutton.PygButton(learnButtonRect, "Learn to Play!")
        learnButton.name = "learnButton"
        self.BUTTONS.append(learnButton)
        
    def handle_Event(self, event, unit=None):        
        for button in self.BUTTONS:
            buttonRetVal = button.handleEvent(event)
            if 'click' in buttonRetVal: #a button has been clicked
                #find what kind of button and act accordingly
                if button.name == 'playButton':
                    return 'play'
                elif button.name == 'learnButton':
                    return 'learn'
                
    def render(self):
        self.screen_Image = pygame.image.load(image_Path + "intro_Image.png")

        for button in self.BUTTONS:
            button.draw(self.screen_Image)
        
        return self.screen_Image

class learn_Screen(Screen):
    """Screen that displays How To info."""
    def __init__(self):
        self.BUTTONS = []
        self.create_Buttons()
    def create_Buttons(self):
        playButtonRect = (650, 550, 125, 25)
        playButton = pygbutton.PygButton(playButtonRect, "Fight!")
        playButton.name = "playButton"
        self.BUTTONS.append(playButton)

    def handle_Event(self, event, unit=None):        
        for button in self.BUTTONS:
            buttonRetVal = button.handleEvent(event)
            if 'click' in buttonRetVal: #a button has been clicked
                #find what kind of button and act accordingly
                if button.name == 'playButton':
                    return 'play'
    def render(self):
        self.screen_Image = pygame.image.load(image_Path + "how_to_play.png")

        for button in self.BUTTONS:
            button.draw(self.screen_Image)
        
        return self.screen_Image

class victory_Screen(Screen):
    """Screen that displays How To info."""
    def __init__(self, winner, loser):
        self.winner = winner
        self.loser = loser
        self.BUTTONS = []
        self.create_Buttons()
    def create_Buttons(self):
        doneButtonRect = (350, 550, 125, 25)
        doneButton = pygbutton.PygButton(doneButtonRect, "Done")
        doneButton.name = "doneButton"
        self.BUTTONS.append(doneButton)

    def handle_Event(self, event, unit=None):        
        for button in self.BUTTONS:
            buttonRetVal = button.handleEvent(event)
            if 'click' in buttonRetVal:
                if button.name == 'doneButton':
                    return 'done'
                
    def render(self):
        self.screen_Image = pygame.image.load(image_Path + "victory.png")

        textFont = pygame.font.Font(None, 60)

        text = "Player " + str(self.loser.number)
        toPrint = textFont.render(text, 1, self.loser.color)
        text_Rect = toPrint.get_rect()
        text_Rect.top = 100
        text_Rect.left = 310
        self.screen_Image.blit(toPrint, text_Rect)

        text = "Congratulations, Player " + str(self.winner.number) + "!"
        toPrint = textFont.render(text, 1, self.winner.color)
        text_Rect = toPrint.get_rect()
        text_Rect.top = 300
        text_Rect.left = 130
        self.screen_Image.blit(toPrint, text_Rect)

        text = "Begin Your Gloating!"
        toPrint = textFont.render(text, 1, self.winner.color)
        text_Rect = toPrint.get_rect()
        text_Rect.top = 370
        text_Rect.left = 170
        self.screen_Image.blit(toPrint, text_Rect)  

        for button in self.BUTTONS:
            button.draw(self.screen_Image)
        
        return self.screen_Image

class preview_Screen(Screen):
    """Screen that displays the preview map."""
    def __init__(self, GameObject):
        self.BUTTONS = []
        self.create_Buttons()

        self.gameObject = GameObject

        self.tileSize = 25

        self.img_grass = pygame.image.load(image_Path + "grass.png")
        self.img_low_hill = pygame.image.load(image_Path + "low_hill.png")
        self.img_mountain = pygame.image.load(image_Path + "mountain.png")
        self.img_rough_grass = pygame.image.load(image_Path + "rough_grass.png")
        self.img_forest = pygame.image.load(image_Path + "forest.png")
        
        self.screen_Image = pygame.Surface((800, 600))
        self.map_Image = None
    def create_Buttons(self):
        doneButtonRect = (350, 570, 125, 25)
        doneButton = pygbutton.PygButton(doneButtonRect, "Done")
        doneButton.name = "doneButton"
        self.BUTTONS.append(doneButton)

    def handle_Event(self, event, unit=None):        
        for button in self.BUTTONS:
            buttonRetVal = button.handleEvent(event)
            if 'click' in buttonRetVal:
                if button.name == 'doneButton':
                    return 'done'
    def draw_Map(self, battle_Map, battle_Map_X, battle_Map_Y):
        """Draws the battle_Map onto a surface."""
        map_Pixel_X = battle_Map_X * self.tileSize
        map_Pixel_Y = battle_Map_Y * self.tileSize
        
        map_Surface = pygame.Surface((map_Pixel_X, map_Pixel_Y))        
        tileImage = pygame.Surface((self.tileSize, self.tileSize))
        
        for y in range(0, battle_Map_X):
            for x in range(0, battle_Map_Y):
                number = battle_Map[x][y]
                if number == 0:
                    tileImage = self.img_grass
                elif number == 1:
                    tileImage = self.img_rough_grass
                elif number == 2:
                    tileImage = self.img_forest
                elif number == 3:
                    tileImage = self.img_low_hill
                else:
                    tileImage = self.img_mountain
                    
                tileRect = pygame.Rect(self.tileSize*(x), self.tileSize*(y), self.tileSize, self.tileSize)

                map_Surface.blit(tileImage, tileRect)
        
        self.map_Image = map_Surface          
    def render(self):
        self.screen_Image.fill(GRAY)

        self.draw_Map(self.gameObject.current_Map, self.gameObject.map_Size, self.gameObject.map_Size)

        map_Rect = (150, 0, 400, 400)

        self.screen_Image.blit(self.map_Image, map_Rect)

        textFont = pygame.font.Font(None, 32)

        text = "Players start at top and bottom. Press space to generate a new map."
        toPrint = textFont.render(text, 1, BLACK)
        text_Rect = toPrint.get_rect()
        text_Rect.top = 500
        text_Rect.left = 30
        self.screen_Image.blit(toPrint, text_Rect)

        text = "When both players are happy with the map, click done."
        toPrint = textFont.render(text, 1, BLACK)
        text_Rect = toPrint.get_rect()
        text_Rect.top = 530
        text_Rect.left = 130
        self.screen_Image.blit(toPrint, text_Rect)

        for button in self.BUTTONS:
            button.draw(self.screen_Image)
        
        return self.screen_Image
    
class map_Screen(Screen):
    """Class that handles the main gameplay screen."""
    def __init__(self, GameObject, player=None, setup=False):
        self.tileSize = 25 #pixel size of individual spaces
        self.mapSize = GameObject.map_Size
        self.player = player
        self.setup = setup

        self.gameObject = GameObject
        
        self.img_panel = pygame.image.load(image_Path + "panel_image.png")
        self.img_info = pygame.image.load(image_Path + "info_box.png")
        
        self.img_dirt = pygame.image.load(image_Path + "dirt.png")
        self.img_grass = pygame.image.load(image_Path + "grass.png")
        self.img_low_hill = pygame.image.load(image_Path + "low_hill.png")
        self.img_mountain = pygame.image.load(image_Path + "mountain.png")
        self.img_rough_grass = pygame.image.load(image_Path + "rough_grass.png")
        self.img_forest = pygame.image.load(image_Path + "forest.png")

        self.img_castle = pygame.image.load(image_Path + "castle.png")

        self.img_Map_Cursor = pygame.image.load(image_Path + "map_cursor.png")

        self.screen_Image = pygame.Surface((800, 600))
        self.map_Image = False

        self.tile_Selected = None
        self.unit_Selected = None

        self.BUTTONS = []

        if setup:
            self.create_Setup_Buttons()
        else:
            self.create_Battle_Buttons()
        #create and initialize buttons
        #Should only need two buttons - "Menu" and "End Turn"
    def create_Setup_Buttons(self):
        """Buttons for the army setup phase."""
        
        doneButtonRect = (650, 550, 75, 25)
        doneButton = pygbutton.PygButton(doneButtonRect, "Done")
        doneButton.name = "doneButton"
        self.BUTTONS.append(doneButton)

    def create_Battle_Buttons(self):
        
        doneButtonRect = (650, 550, 75, 25)
        doneButton = pygbutton.PygButton(doneButtonRect, "Done")
        doneButton.name = "doneButton"
        self.BUTTONS.append(doneButton)
        
    def handle_Event(self, event, unit=None):
        if self.setup:  #---------------SETUP PHASE-----------------
            self.unit_Selected = unit
            if event.type == MOUSEBUTTONUP and event.button == 1:
                if self.mouse_In_Map(event.pos):
                    self.tile_Selected = self.mouse_To_Map(event.pos)
                    if unit == None:
                        pass
                    elif unit == 'castle':
                        if self.is_Legal_Placement(self.tile_Selected):
                            self.player.set_Castle_Location(self.tile_Selected)
                    else:
                        if self.is_Legal_Placement(self.tile_Selected):
                            unit.set_Location(self.tile_Selected)
            for button in self.BUTTONS:
                buttonRetVal = button.handleEvent(event)
                if 'click' in buttonRetVal: 
                    if button.name == 'doneButton':
                        return 'done'
        else: #---------------BATTLE PHASE----------------------
            if event.type == MOUSEBUTTONUP and event.button == 1:
                if self.mouse_In_Map(event.pos):
                    self.tile_Selected = self.mouse_To_Map(event.pos)
                    if not self.unit_Selected == None and self.unit_Selected.color == self.player.color:
                        checkTile = self.is_Valid_Move(self.tile_Selected)
                        if checkTile == False:
                            #not a valid move
                            self.unit_Selected = None
                        elif not type(checkTile) == int and self.unit_Selected.color == self.player.color:
                            #a valid attack
                            defender = checkTile
                            terrain = self.gameObject.current_Map[self.tile_Selected[0]][self.tile_Selected[1]]
                            defender.defend(self.unit_Selected, ctc_terrain.ALL_TERRAIN[terrain])                          
                            self.unit_Selected.valid_Moves = None
                            self.unit_Selected = None
                            self.tile_Selected = None
                        elif type(checkTile) == int and self.unit_Selected.color == self.player.color:
                            #a valid move
                            self.unit_Selected.location = self.tile_Selected
                            self.unit_Selected.current_Move -= checkTile
                            self.unit_Selected.valid_Moves = None
                            self.unit_Selected = None
                            self.tile_Selected = None
                    elif self.unit_Selection():
                        if self.unit_Selected.valid_Moves == None and self.unit_Selected.color == self.player.color:
                            self.unit_Selected.determine_Valid_Moves(self.gameObject.current_Map) 
            for button in self.BUTTONS:
                buttonRetVal = button.handleEvent(event)
                if 'click' in buttonRetVal: #a button has been clicked
                    #find what kind of button and act accordingly
                    if button.name == 'doneButton':
                        return 'done'
                    
    def unit_Selection(self):
        """Sets unit_Selected if appropriate, returns True if is set."""
        player1 = self.gameObject.player_1
        player2 = self.gameObject.player_2
        p1_Units = self.gameObject.player_1.ALL_UNITS
        p2_Units = self.gameObject.player_2.ALL_UNITS

        ALL_UNITS = p1_Units + p2_Units

        for unit in ALL_UNITS:
            if unit == None:
                print "Something is fucked up."
            if unit.location == self.tile_Selected:
                self.unit_Selected = unit
                return True

        self.unit_Selected = None
        return False

    def draw_Map(self, battle_Map, battle_Map_X, battle_Map_Y):
        """Draws the battle_Map onto a surface."""
        map_Pixel_X = battle_Map_X * self.tileSize
        map_Pixel_Y = battle_Map_Y * self.tileSize
        
        map_Surface = pygame.Surface((map_Pixel_X, map_Pixel_Y))        
        tileImage = pygame.Surface((self.tileSize, self.tileSize))
        
        for y in range(0, battle_Map_X):
            for x in range(0, battle_Map_Y):
                number = battle_Map[x][y]
                if number == 0:
                    tileImage = self.img_grass
                elif number == 1:
                    tileImage = self.img_rough_grass
                elif number == 2:
                    tileImage = self.img_forest
                elif number == 3:
                    tileImage = self.img_low_hill
                else:
                    tileImage = self.img_mountain
                    
                tileRect = pygame.Rect(self.tileSize*(x), self.tileSize*(y), self.tileSize, self.tileSize)

                map_Surface.blit(tileImage, tileRect)
        
        self.map_Image = map_Surface
    def render(self, current_Map):
        self.screen_Image.blit(self.img_panel, self.img_panel.get_rect())

        textFont = pygame.font.Font(None, 32)

        info_Box_Rectangle = pygame.Rect(525, 50, 250, 500)
        self.screen_Image.blit(self.img_info, info_Box_Rectangle)

        self.draw_Map(current_Map, self.mapSize, self.mapSize)
        
        if self.map_Image:
            screen_Rec = pygame.Rect(0, (self.tileSize*2), 800, 600)
            self.screen_Image.blit(self.map_Image, screen_Rec)

        #This code only applicable to setup phase
        if self.setup:
            if self.player.number == 1:
                field_Rect = pygame.Rect(0, 50, 500, 75)
            else:
                field_Rect = pygame.Rect(0, 475, 500, 75)

            if self.player.castle_Placed == False:
                text = "Place your Castle. Click 'Done' when ready to place units."
                toPrint = textFont.render(text, 1, self.player.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 15
                text_Rect.left = 15
                self.screen_Image.blit(toPrint, text_Rect)
            elif not self.player.all_Units_Placed():
                text = "Place your next Unit"
                toPrint = textFont.render(text, 1, self.player.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 15
                text_Rect.left = 15
                self.screen_Image.blit(toPrint, text_Rect)

                if not self.unit_Selected == None and not type(self.unit_Selected) == str:
                    #info box stuff                    
                    unit_Info_Icon = self.unit_Selected.icon
                    unit_Info_Icon_Rect = (535, 60, 50, 50)
                    self.screen_Image.blit(unit_Info_Icon, unit_Info_Icon_Rect)

                    text = str(self.unit_Selected.unit_Type.type_Name)
                    toPrint = textFont.render(text, 1, self.player.color)
                    text_Rect = toPrint.get_rect()
                    text_Rect.top = 150
                    text_Rect.left = 535
                    self.screen_Image.blit(toPrint, text_Rect)

                    text = "Move: " + str(self.unit_Selected.current_Move)
                    toPrint = textFont.render(text, 1, self.player.color)
                    text_Rect = toPrint.get_rect()
                    text_Rect.top = 180
                    text_Rect.left = 535
                    self.screen_Image.blit(toPrint, text_Rect)

                    text = "Soldiers: " + str(self.unit_Selected.soldiers)
                    toPrint = textFont.render(text, 1, self.player.color)
                    text_Rect = toPrint.get_rect()
                    text_Rect.top = 210
                    text_Rect.left = 535
                    self.screen_Image.blit(toPrint, text_Rect)

                    text = "Attack: " + str(self.unit_Selected.unit_Type.attack)
                    toPrint = textFont.render(text, 1, self.player.color)
                    text_Rect = toPrint.get_rect()
                    text_Rect.top = 240
                    text_Rect.left = 535
                    self.screen_Image.blit(toPrint, text_Rect)

                    text = "Armor: " + str(self.unit_Selected.unit_Type.armor)
                    toPrint = textFont.render(text, 1, self.player.color)
                    text_Rect = toPrint.get_rect()
                    text_Rect.top = 270
                    text_Rect.left = 535
                    self.screen_Image.blit(toPrint, text_Rect)
                
            else:
                text = "All units placed! Hit 'Done' to end setup phase."
                toPrint = textFont.render(text, 1, self.player.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 15
                text_Rect.left = 15
                self.screen_Image.blit(toPrint, text_Rect)             

            setup_Field = pygame.Surface((500, 75))
            setup_Field.fill(self.player.color)
            setup_Field.set_alpha(100)

            self.screen_Image.blit(setup_Field, field_Rect)

            if not self.player.castle_Location == None:
                castle_Rect = ((self.player.castle_Location[0]*25), ((self.player.castle_Location[1]*25)+50), 25, 25)
                self.screen_Image.blit(self.img_castle, castle_Rect)

            for unit in self.player.ALL_UNITS:
                if not unit.location == None:
                    unit_Rect = ((unit.location[0]*25), ((unit.location[1]*25)+50), 25, 25)
                    unit_Image = unit.icon_Small
                    self.screen_Image.blit(unit_Image, unit_Rect)
            
        #END setup phase code

        #BEGIN battle phase code
        if not self.setup:
            player1 = self.gameObject.player_1
            player2 = self.gameObject.player_2
            p1_Units = self.gameObject.player_1.ALL_UNITS
            p2_Units = self.gameObject.player_2.ALL_UNITS

            ALL_UNITS = p1_Units + p2_Units

            text = "Player " + str(self.player.number) + ", it is your turn!"
            toPrint = textFont.render(text, 1, self.player.color)
            text_Rect = toPrint.get_rect()
            text_Rect.top = 15
            text_Rect.left = 15
            self.screen_Image.blit(toPrint, text_Rect)            

            #blit castles
            p1_Castle_Rect = ((player1.castle_Location[0]*25), ((player1.castle_Location[1]*25)+50), 25, 25)
            p2_Castle_Rect = ((player2.castle_Location[0]*25), ((player2.castle_Location[1]*25)+50), 25, 25)
            self.screen_Image.blit(self.img_castle, p1_Castle_Rect)
            self.screen_Image.blit(self.img_castle, p2_Castle_Rect)

            #blit player units
            for unit in ALL_UNITS:
                if not unit.soldiers == 0:
                    unit_Rect = ((unit.location[0]*25), ((unit.location[1]*25)+50), 25, 25)
                    unit_Image = unit.icon_Small
                    self.screen_Image.blit(unit_Image, unit_Rect)

            #if unit selected, blit the possible moves
            if not self.unit_Selected == None and not self.unit_Selected.valid_Moves == None:
                for tile in self.unit_Selected.valid_Moves:
                    tile_Rect = ((tile.location[0]*25), ((tile.location[1]*25)+50), 25, 25)
                    tile_Image = pygame.Surface((25, 25))
                    tile_Image.fill(self.player.color)
                    tile_Image.set_alpha(100)
                    self.screen_Image.blit(tile_Image, tile_Rect)

            #blit info box stuff
            if not self.unit_Selected == None:
                unit_Info_Icon = self.unit_Selected.icon
                unit_Info_Icon_Rect = (535, 60, 50, 50)
                self.screen_Image.blit(unit_Info_Icon, unit_Info_Icon_Rect)

                #text
                text = str(self.unit_Selected.unit_Type.type_Name)
                toPrint = textFont.render(text, 1, self.unit_Selected.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 150
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

                text = "Move: " + str(self.unit_Selected.current_Move)
                toPrint = textFont.render(text, 1, self.unit_Selected.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 180
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

                text = "Soldiers: " + str(self.unit_Selected.soldiers)
                toPrint = textFont.render(text, 1, self.unit_Selected.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 210
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

                text = "Attack: " + str(self.unit_Selected.unit_Type.attack)
                toPrint = textFont.render(text, 1, self.unit_Selected.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 240
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

                text = "Armor: " + str(self.unit_Selected.unit_Type.armor)
                toPrint = textFont.render(text, 1, self.unit_Selected.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 270
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

            if not self.tile_Selected == None:
                number = self.gameObject.current_Map[self.tile_Selected[0]][self.tile_Selected[1]]
                if number == 0:
                    tileImage = self.img_grass
                elif number == 1:
                    tileImage = self.img_rough_grass
                elif number == 2:
                    tileImage = self.img_forest
                elif number == 3:
                    tileImage = self.img_low_hill
                else:
                    tileImage = self.img_mountain
                tileRect = (535, 310, 25, 25)
                self.screen_Image.blit(tileImage, tileRect)
                
                terrain = ctc_terrain.ALL_TERRAIN[number]
                
                text = terrain.type_Name
                toPrint = textFont.render(text, 1, self.player.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 340
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

                text = "Move Cost: " + str(terrain.move_Pen)
                toPrint = textFont.render(text, 1, self.player.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 370
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

                text = "Range Bonus: " + str(terrain.range_Bon)
                toPrint = textFont.render(text, 1, self.player.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 400
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

                text = "Defense Bonus: " + str(terrain.armor_Bon)
                toPrint = textFont.render(text, 1, self.player.color)
                text_Rect = toPrint.get_rect()
                text_Rect.top = 430
                text_Rect.left = 535
                self.screen_Image.blit(toPrint, text_Rect)

        #END battle phase code

        if not self.tile_Selected == None:
            tile = self.tile_Selected
            tileRect = pygame.Rect((self.tileSize*tile[0]), (self.tileSize*tile[1]+50), self.tileSize, self.tileSize)
            self.screen_Image.blit(self.img_Map_Cursor, tileRect)
            
        for button in self.BUTTONS:
            button.draw(self.screen_Image)
        
        return self.screen_Image
    def mouse_In_Map(self, position):
        """Returns true if position is within map; false otherwise."""

        map_X_Min = 0
        map_X_Max = 500
        map_Y_Min = 50
        map_Y_Max = 550
        
        if position[0] < map_X_Min or position[0] >= map_X_Max:
            return False
        elif position[1] < map_Y_Min or position[1] >= map_Y_Max:
            return False
        else:
            return True
    def is_Legal_Placement(self, tile):        
        x_Min = 0
        x_Max = 19

        if self.player.number == 1:
            y_Min = 0
            y_Max = 2
        else:
            y_Min = 17
            y_Max = 19

        if tile[0] < x_Min or tile[0] > x_Max or \
           tile[1] < y_Min or tile[1] > y_Max:
            return False

        if self.player.castle_Location == tile:
            return False

        for unit in self.player.ALL_UNITS:
            if unit.location == tile:
                return False
            
        return True
    def is_Valid_Move(self, tile):
        """Determines if tile is a valid move for self.unit_Selected.

        If move is valid, function returns the path cost. Else, returns false."""

        unit = self.unit_Selected
        moves = unit.valid_Moves

        #check if enemy unit exists at that spot
        p1_Units = self.gameObject.player_1.ALL_UNITS
        p2_Units = self.gameObject.player_2.ALL_UNITS

        ALL_UNITS = p1_Units + p2_Units

        #cases:
        #unit in location and is friend - can't move, can't attack
        #unit in location and is enemy and is adjacent - can't move, can attack

        unit_There = False
        is_Enemy = False
        for unit in ALL_UNITS:
            if unit.location == tile:
                unit_There = True
                if not unit.color == self.player.color:
                    is_Enemy = True
                    enemy_Unit = unit
                    break

        #check if adjacent
        attack_Location = self.unit_Selected.location
        defend_Location = tile

        diff_X = attack_Location[0] - defend_Location[0]
        diff_Y = attack_Location[1] - defend_Location[1]

        if abs(diff_X) > 1 or abs(diff_Y) > 1:
            adjacent = False
        else:
            adjacent = True

        #check if move remaining
        if self.unit_Selected.current_Move > 0:
            move_Remaining = True
        else:
            move_Remaining = False

        if is_Enemy and adjacent and move_Remaining:
            return enemy_Unit
        elif unit_There == False:
            for move in moves:
                if move.location == tile:
                    return move.path_Cost
        else:
            return False
    def mouse_To_Map(self, position):
        mouse_X = position[0]
        mouse_Y = position[1]

        map_X = mouse_X / 25
        map_Y = (mouse_Y-50) / 25

        return (map_X, map_Y)

class army_Screen(Screen):
    """Class that handles the main gameplay screen."""
    
    def __init__(self, GameObject, player):
        #Buttons needed:
        #Clear Army
        #One button for each troop type (click to add soldier)
        #New buttons for each unit in army (click to remove soldier)
        #Accept Army
        #View Map

        self.screen_Image = pygame.Surface((800, 600))
        self.gameObject = GameObject

        self.player = player
        self.selected_Slot = 0 #which army slot the cursor is over

        self.ARMY_SLOT_BUTTONS = []
        self.UNIT_TYPE_BUTTONS = []
        self.OTHER_BUTTONS = []

        self.notice = False #true if an error needs to be displayed
        self.notice_Test = None #will contain text to display

        self.create_Army_Slots()
        self.create_Unit_Buttons(GameObject)
        self.create_Other_Buttons()
        
    def create_Army_Slots(self):
        highlight_image = pygame.image.load(image_Path + "map_cursor.png")
        highlight_image = pygame.transform.smoothscale(highlight_image, (50,50))
        
        button_X_Min, button_X = 100, 100
        button_Y_Min, button_Y = 75, 75

        button_X_Max = 700
        button_Y_Max = 500
        position_Step = 100

        button_Size = 50
        
        ARMY_SLOT_BUTTONS = []
        slot_Index = 0
        for unit in self.player.ALL_UNITS:
            #determine the Rect stats
            b_rect = (button_X, button_Y, button_Size, button_Size)

            if unit == None:
                slot_image = pygame.Surface((50, 50))
                slot_image.fill(GRAY)
                slot_highlight = pygame.Surface((50, 50))
                slot_highlight.fill(LIGHTGRAY)
            else:
                slot_image = pygame.Surface((50, 50))
                slot_image.fill(GRAY)
                slot_image.blit(unit.icon, unit.icon.get_rect())
                
                slot_highlight = pygame.Surface((50, 50))
                slot_highlight.blit(unit.icon, unit.icon.get_rect())
                slot_highlight.blit(highlight_image, slot_highlight.get_rect())

            pygame.draw.line(slot_image, BLACK, (0, 0), (0, 49))
            pygame.draw.line(slot_image, BLACK, (0, 0), (49, 0))

            pygame.draw.line(slot_image, LIGHTGRAY, (49, 49), (0, 49))
            pygame.draw.line(slot_image, LIGHTGRAY, (49, 49), (49, 0))

            button = pygbutton.ctc_ArmySlotButton(rect=b_rect, normal=slot_image, highlight=slot_highlight)
             
            #all slots start out empty, but button.unit will point to
            #the unit represented in the slot when full and actions
            #will affect it
            button.name = 'slot_button'
            button.unit = unit
            button.slot_Index = slot_Index
            ARMY_SLOT_BUTTONS.append(button)

            slot_Index += 1
            button_Y += position_Step

            if button_Y > button_Y_Max:
                button_Y = button_Y_Min
                button_X += position_Step

        self.ARMY_SLOT_BUTTONS = ARMY_SLOT_BUTTONS
        
    def create_Unit_Buttons(self, GameObject):
        """Creates the buttons for each unit type."""
        button_X_Min, button_X = 400, 400
        button_Y_Min, button_Y = 75, 75

        button_X_Max = 700
        button_Y_Max = 500
        position_Step = 75

        button_Size = 50
        
        UNIT_TYPE_BUTTONS = []
        for unit in GameObject.UNIT_TYPES:

            soldier_image = pygame.Surface((50, 50))
            soldier_image.fill(GRAY)
            soldier_image.blit(unit.icon, unit.icon.get_rect())
        
            highlight_image = pygame.image.load(image_Path + "map_cursor.png")
            highlight_image = pygame.transform.smoothscale(highlight_image, (50,50))
        
            soldier_highlight = pygame.Surface((50, 50))
            soldier_highlight.blit(unit.icon, unit.icon.get_rect())
            soldier_highlight.blit(highlight_image, soldier_highlight.get_rect())

            #determine the Rect stats
            b_rect = (button_X, button_Y, button_Size, button_Size)
            button = pygbutton.PygButton(rect=b_rect, normal=soldier_image, highlight=soldier_highlight)

            #link button to unit it represents
            button.name = 'unit_type_button'
            button.unit_Type = unit
            UNIT_TYPE_BUTTONS.append(button)

            button_X += position_Step

            if button_X > button_X_Max:
                button_X = button_X_Min
                button_Y += position_Step

        self.UNIT_TYPE_BUTTONS = UNIT_TYPE_BUTTONS
    def create_Other_Buttons(self):
        """Creates non unit-related functionality, such as viewing the map."""
        doneButtonRect = (650, 550, 75, 25)
        doneButton = pygbutton.PygButton(doneButtonRect, "Done")
        doneButton.name = "doneButton"
        self.OTHER_BUTTONS.append(doneButton)
        
        clearArmyButtonRect = (100, 550, 100, 25)
        clearArmyButton = pygbutton.PygButton(clearArmyButtonRect, "Clear Army")
        clearArmyButton.name = "clearArmyButton"
        self.OTHER_BUTTONS.append(clearArmyButton)
        
    def handle_Event(self, event):

        army_Changed = False
        ALL_BUTTONS = self.UNIT_TYPE_BUTTONS + self.ARMY_SLOT_BUTTONS + self.OTHER_BUTTONS
        for button in ALL_BUTTONS:
            buttonRetVal = button.handleEvent(event)
            if 'click' in buttonRetVal: #a button has been clicked
                #find what kind of button and act accordingly
                if button.name == 'slot_button':
                    if not button.unit == None and \
                       self.selected_Slot == button.slot_Index:
                        army_Changed = True
                        self.player.remove_Unit(self.selected_Slot)
                    else:
                        self.selected_Slot = button.slot_Index
                elif button.name == 'unit_type_button':
                    army_Changed = True
                    addRetVal = self.player.add_Unit(self.selected_Slot, button.unit_Type)
                    if type( addRetVal ) == int:
                        self.selected_Slot = addRetVal
                elif button.name == 'doneButton':
                    #check if the player actually has an army
                    if self.player.army_Not_Empty():
                        return 'done'
                    else:
                        self.notice = True
                        self.notice_Text = "You must have at least one soldier in your army!"                
                elif button.name == 'clearArmyButton':
                    self.player.clear_Army()
                    army_Changed = True

        if army_Changed:
            self.create_Army_Slots()

    def render(self):
        #print standard text
        self.screen_Image.fill(GRAY)
        textFont = pygame.font.Font(None, 32)
        text = "Player " + str(self.player.number) + ", you have " + \
               str(self.player.points) + " Army Points remaining"
        toPrint = textFont.render(text, 1, self.player.color)
        self.screen_Image.blit(toPrint, toPrint.get_rect())

        #print notice text, if any
        if self.notice:
            if type(self.notice) == int:
                self.notice -= 1 #only display it once
                if self.notice == 0:
                    self.notice = False
            else:
                self.notice = 100
                
            noticePrint = textFont.render(self.notice_Text, 1, BLACK)
            noticeRect = noticePrint.get_rect().copy()
            noticeRect.top += 30
            self.screen_Image.blit(noticePrint, noticeRect)

        ALL_BUTTONS = self.UNIT_TYPE_BUTTONS + self.ARMY_SLOT_BUTTONS + self.OTHER_BUTTONS

        for button in ALL_BUTTONS:
            button.draw(self.screen_Image)

        outline = pygame.image.load(image_Path + "box_outline.png")
        unit = None
        for index, button in enumerate(self.ARMY_SLOT_BUTTONS):
            if index == self.selected_Slot:
                rect = button.rect.copy()
                rect.top -= 1
                rect.left -= 1
                self.screen_Image.blit(outline, rect)
                unit = button.unit

        if not unit == None:
            text = "Unit Type: " + str(unit.unit_Type.type_Name)
            toPrint = textFont.render(text, 1, self.player.color)
            text_Rect = toPrint.get_rect()
            text_Rect.top = 300
            text_Rect.left = 400
            self.screen_Image.blit(toPrint, text_Rect)

            text = "Move: " + str(unit.current_Move)
            toPrint = textFont.render(text, 1, self.player.color)
            text_Rect = toPrint.get_rect()
            text_Rect.top = 330
            text_Rect.left = 400
            self.screen_Image.blit(toPrint, text_Rect)

            text = "Soldiers: " + str(unit.soldiers)
            toPrint = textFont.render(text, 1, self.player.color)
            text_Rect = toPrint.get_rect()
            text_Rect.top = 360
            text_Rect.left = 400
            self.screen_Image.blit(toPrint, text_Rect)

            text = "Attack: " + str(unit.unit_Type.attack)
            toPrint = textFont.render(text, 1, self.player.color)
            text_Rect = toPrint.get_rect()
            text_Rect.top = 390
            text_Rect.left = 400
            self.screen_Image.blit(toPrint, text_Rect)

            text = "Armor: " + str(unit.unit_Type.armor)
            toPrint = textFont.render(text, 1, self.player.color)
            text_Rect = toPrint.get_rect()
            text_Rect.top = 420
            text_Rect.left = 400
            self.screen_Image.blit(toPrint, text_Rect)

        return self.screen_Image            
