#Placeholder Unit classes to use for development
#Will be replaced by Iverson's class

import pygame
from pygame.locals import *

import ctc_terrain
from ctc_imports import image_Path

#image_Path = "C:\\Python27\\ctc\\images\\"

class PathStep:
    """Container class used by the pathfinding code."""
    def __init__(self, path_Cost, location):
        self.location = location
        self.path_Cost = path_Cost

class Unit_Type:
    """Contains the basic stats for various kinds of army units."""
    def __init__(self, type_Name, armor, attack, move, health=2, \
                 point_Cost=1, icon_Name=None):
        """For each new unit_Type armor, attack, and move value are needed."""

        self.type_Name = type_Name
        self.armor = armor
        self.attack = attack
        self.move = move
        self.health = health
        self.cost = point_Cost

        if icon_Name == None:
            self.icon_Path = "basic_soldier.png"

        #need to error handle the missing image case
        self.icon = pygame.image.load(image_Path + "blank.png")
        self.soldier = pygame.image.load(image_Path + icon_Name)
        self.icon.blit(self.soldier, self.soldier.get_rect())

class Unit:
    """Virtual base class for CTC Units."""
    def __init__(self, unit_Type, color):
        """Creates an initial unit with one soldier and the player's color."""
        self.unit_Type = unit_Type
        self.color = color
        self.soldiers = 1
        self.current_Health = unit_Type.health
        self.current_Move = unit_Type.move

        self.location = None #eventually a (x, y) coord pair
        self.valid_Moves = None #eventually a list of coord pairs

        self.icon = pygame.image.load(image_Path + "blank.png")
        self.icon.blit(self.unit_Type.icon, self.unit_Type.icon.get_rect())

        hat_band = pygame.Rect(15, 11, 23, 5)
        shirt = pygame.Rect(21, 34, 10, 10)

        self.icon.fill(self.color, hat_band)
        self.icon.fill(self.color, shirt)

        self.icon_Small = pygame.transform.smoothscale(self.icon, (25, 25))
    def set_Location(self, location):
        unit_X = location[0]
        unit_Y = location[1]

        self.location = (unit_X, unit_Y)

    def determine_Valid_Moves(self, battle_Map):
        """Sets the list of valid moves for this unit.

        List is generated through simple best-first path exploration."""

        valid_Moves = []

        if self.location == None:
            return valid_Moves

        #don't really need to remember path, just total path cost.
        #thus each "step" just needs to remember total cost-so-far
        self.explored = []
        #self.explored_Location_Only = []
        self.unexplored = []
        self.unexplored.append(PathStep(0, self.location))

        moves_Remaining = True

        while moves_Remaining:
            self.unexplored = sorted(self.unexplored, key=lambda PathStep: PathStep.path_Cost, reverse=True)
            #for item in self.unexplored:
            #    print item.location, item.path_Cost
            #print "----------"
            current = self.unexplored.pop()

            if current.path_Cost <= self.current_Move and \
               not self.location_In_List(current.location, self.explored):
                self.add_Neighbors(current, battle_Map)
                if current.path_Cost > 0:
                    self.explored.append(current)
            else:
                moves_Remaining = False

        self.valid_Moves = self.explored
        #for item in self.valid_Moves:
            #print item.location, item.path_Cost

    def add_Neighbors(self, current, battle_Map):
        """Creates PathStep objects for neighbors and adds to unexplored."""
        map_Min = 0
        map_Max = 20

        current_Cost = current.path_Cost
        current_Loc = current.location

        #add all neighbors if it's an actual tile and
        #if it hasn't been added already

        #the four cardinal directions (no extra path cost)
        neighbor = (current_Loc[0], current_Loc[1]-1)
        if neighbor[0] < map_Min or neighbor[1] < map_Min or \
           neighbor[0] >= map_Max or neighbor[1] >= map_Max:
            pass
        elif self.location_In_List(neighbor, self.explored) or \
             self.location_In_List(neighbor, self.unexplored):
            pass
        else:
            map_Tile = battle_Map[neighbor[0]][neighbor[1]]
            path_Cost = current_Cost + ctc_terrain.ALL_TERRAIN[map_Tile].move_Pen + 1
            self.unexplored.append(PathStep(path_Cost, neighbor))
            
        neighbor = (current_Loc[0], current_Loc[1]+1)
        if neighbor[0] < map_Min or neighbor[1] < map_Min or \
           neighbor[0] >= map_Max or neighbor[1] >= map_Max:
            pass
        elif self.location_In_List(neighbor, self.explored) or \
             self.location_In_List(neighbor, self.unexplored):
            pass
        else:
            map_Tile = battle_Map[neighbor[0]][neighbor[1]]
            path_Cost = current_Cost + ctc_terrain.ALL_TERRAIN[map_Tile].move_Pen + 1
            self.unexplored.append(PathStep(path_Cost, neighbor))

        neighbor = (current_Loc[0]+1, current_Loc[1])
        if neighbor[0] < map_Min or neighbor[1] < map_Min or \
           neighbor[0] >= map_Max or neighbor[1] >= map_Max:
            pass
        elif self.location_In_List(neighbor, self.explored) or \
             self.location_In_List(neighbor, self.unexplored):
            pass
        else:
            map_Tile = battle_Map[neighbor[0]][neighbor[1]]
            path_Cost = current_Cost + ctc_terrain.ALL_TERRAIN[map_Tile].move_Pen + 1
            self.unexplored.append(PathStep(path_Cost, neighbor))

        neighbor = (current_Loc[0]-1, current_Loc[1])
        if neighbor[0] < map_Min or neighbor[1] < map_Min or \
           neighbor[0] >= map_Max or neighbor[1] >= map_Max:
            pass
        elif self.location_In_List(neighbor, self.explored) or \
             self.location_In_List(neighbor, self.unexplored):
            pass
        else:
            map_Tile = battle_Map[neighbor[0]][neighbor[1]]
            path_Cost = current_Cost + ctc_terrain.ALL_TERRAIN[map_Tile].move_Pen + 1
            self.unexplored.append(PathStep(path_Cost, neighbor))

        #the four diagonals (one additional path cost)
        neighbor = (current_Loc[0]-1, current_Loc[1]-1)
        if neighbor[0] < map_Min or neighbor[1] < map_Min or \
           neighbor[0] >= map_Max or neighbor[1] >= map_Max:
            pass
        elif self.location_In_List(neighbor, self.explored) or \
             self.location_In_List(neighbor, self.unexplored):
            pass
        else:
            map_Tile = battle_Map[neighbor[0]][neighbor[1]]
            path_Cost = current_Cost + ctc_terrain.ALL_TERRAIN[map_Tile].move_Pen + 2
            self.unexplored.append(PathStep(path_Cost, neighbor))
            
        neighbor = (current_Loc[0]+1, current_Loc[1]+1)
        if neighbor[0] < map_Min or neighbor[1] < map_Min or \
           neighbor[0] >= map_Max or neighbor[1] >= map_Max:
            pass
        elif self.location_In_List(neighbor, self.explored) or \
             self.location_In_List(neighbor, self.unexplored):
            pass
        else:
            map_Tile = battle_Map[neighbor[0]][neighbor[1]]
            path_Cost = current_Cost + ctc_terrain.ALL_TERRAIN[map_Tile].move_Pen + 2
            self.unexplored.append(PathStep(path_Cost, neighbor))
        
        neighbor = (current_Loc[0]+1, current_Loc[1]-1)
        if neighbor[0] < map_Min or neighbor[1] < map_Min or \
           neighbor[0] >= map_Max or neighbor[1] >= map_Max:
            pass
        elif self.location_In_List(neighbor, self.explored) or \
             self.location_In_List(neighbor, self.unexplored):
            pass
        else:
            map_Tile = battle_Map[neighbor[0]][neighbor[1]]
            path_Cost = current_Cost + ctc_terrain.ALL_TERRAIN[map_Tile].move_Pen + 2
            self.unexplored.append(PathStep(path_Cost, neighbor))
            
        neighbor = (current_Loc[0]-1, current_Loc[1]+1)
        if neighbor[0] < map_Min or neighbor[1] < map_Min or \
           neighbor[0] >= map_Max or neighbor[1] >= map_Max:
            pass
        elif self.location_In_List(neighbor, self.explored) or \
             self.location_In_List(neighbor, self.unexplored):
            pass
        else:
            map_Tile = battle_Map[neighbor[0]][neighbor[1]]
            path_Cost = current_Cost + ctc_terrain.ALL_TERRAIN[map_Tile].move_Pen + 2
            self.unexplored.append(PathStep(path_Cost, neighbor))
    
    def location_In_List(self, location, the_List):
        """Returns True if no PathStep in the_List includes location."""
        for pathStep in the_List:
            if pathStep.location == location:
                return True

        return False

    def defend(self, attacker, terrain):
        """Applies combat equations to both self and attacker."""
        attacker_Score = attacker.unit_Type.attack * attacker.soldiers
        defender_Score = (self.unit_Type.armor + terrain.armor_Bon) * self.soldiers

        attacker_Score -= defender_Score
        if attacker_Score < 1:
            attacker_Score = 1

        health_Damage = attacker_Score

        soldiers_Killed = health_Damage/self.unit_Type.health
        if soldiers_Killed < 1:
            soldiers_Killed = 1

        self.soldiers -= soldiers_Killed

        if self.soldiers <= 0:
            self.location = None

#archer = Unit_Type('archer', 1, 1, 5, point_Cost = 2, icon_Name="archer.png")
spearman = Unit_Type('spearman', 3, 1, 4, point_Cost = 3, icon_Name="spearman.png")
infantry = Unit_Type('axeman', 1, 2, 5, point_Cost = 3, icon_Name="infantry.png")
cavalry = Unit_Type('cavalry', 1, 2, 7, point_Cost = 5, icon_Name="cavalry.png")

UNIT_TYPES = [spearman, infantry, cavalry]
