#ctc_game - overarching game controller for Conquer the Castle

import pygame
from pygame.locals import *

from ctc_imports import *
import copy

class ctc_Game():
    """Top-level class for the game and location of the main loop."""
    def __init__(self):
        pygame.init()
        pygame.mixer.init()

        if pygame.mixer.get_init() == None:
            sound = True
        else:
            sound = False

        #intro screen display
        RED  = (200, 0, 0)
        BLUE = (0, 0, 200)
        
        self.player_1 = ctc_player.Player(RED, 1)
        self.player_2 = ctc_player.Player(BLUE, 2)
        
        self.UNIT_TYPES = ctc_unit.UNIT_TYPES
        self.map_Size = 20

        self.mapper = ctc_mapgen.MapGenerator()
        self.render = ctc_render.Draw()
        
        self.current_Map = self.mapper.new_Map(self.map_Size, self.map_Size)

        #for row in self.current_Map:
        #    print row
        
        #should briefly display some sort of loading screen
        #probably won't have time to register, though
    def new_Map(self):
        self.current_Map = self.mapper.new_Map(self.map_Size, self.map_Size)
    def run(self):
        """Top-view of the course of a game."""
        self.render.init_Window()

        response = self.intro()

        if response == 'learn':
            self.learn()

        self.map_Preview()
        
        self.build_Army(self.player_1)
        self.build_Army(self.player_2)

        self.place_Units(self.player_1)
        self.place_Units(self.player_2)

        #might want a "place units" screen here. It would use the same
        #base setup as the map screen (drawn from the same screen)
        #but has different input handling
        
        #Go to Main Gameplay Screen
        winner = self.fight_Battle()

        self.victory(winner)
    def intro(self):
        """Displays intro screen."""
        intro_Screen = ctc_screen.intro_Screen()
        self.render.draw_Screen(intro_Screen.render())

        ready = False

        while not ready:
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                else:
                    retVal = intro_Screen.handle_Event(event)
                    if retVal == "play":
                        return retVal
                    elif retVal == 'learn':
                        return retVal

            self.render.draw_Screen(intro_Screen.render())
            
    def learn(self):
        """Displays how-to-play screen."""
        learn_Screen = ctc_screen.learn_Screen()
        self.render.draw_Screen(learn_Screen.render())

        ready = False

        while not ready:
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                else:
                    retVal = learn_Screen.handle_Event(event)
                    if retVal == "play":
                        ready = True                    

            self.render.draw_Screen(learn_Screen.render())
    def map_Preview(self):
        """Lets players preview and regenerate the map."""
        preview_Screen = ctc_screen.preview_Screen(self)
        self.render.draw_Screen(preview_Screen.render())

        ready = False

        while not ready:
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                elif event.type == KEYUP and event.key == K_SPACE:
                    self.new_Map()
                else:
                    retVal = preview_Screen.handle_Event(event)
                    if retVal == "done":
                        ready = True                    

            self.render.draw_Screen(preview_Screen.render())
        
    def build_Army(self, player):
        """Handles the army building screen input/actions."""
        army_Screen = ctc_screen.army_Screen(self, player)
        self.render.draw_Screen(army_Screen.render())

        player_Done = False

        while not player_Done:
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                else:
                    retVal = army_Screen.handle_Event(event)
                    if retVal == "done":
                        player_Done = True
                    else:
                        self.render.draw_Screen(army_Screen.render())
                        
    def place_Units(self, player):
        """Players place their armies upon the map."""
        player.trim_Unit_List()
        
        map_Screen = ctc_screen.map_Screen(self, player, True)
        self.render.draw_Screen(map_Screen.render(self.current_Map))

        player_Done = False
        castle_Placed = False
        all_Units_Placed = False
        place_Next_Unit = True
        
        while not player_Done:
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                else:
                    if not castle_Placed:
                        retVal = map_Screen.handle_Event(event, 'castle')
                        if retVal == 'done':
                            player.castle_Placed = True
                            castle_Placed = True
                    elif not all_Units_Placed:
                        if place_Next_Unit:
                            place_Next_Unit = False
                            next_Unit = player.next_Unplaced_Unit()
                            if next_Unit == False:
                                all_Units_Placed = True
                                player_Done = True
                                continue                            
                        retVal = map_Screen.handle_Event(event, next_Unit)
                        if retVal == 'done':
                            place_Next_Unit = True
                            empty_Event = pygame.event.Event(USEREVENT, {})
                            pygame.event.post(empty_Event)
                            
                            
                    #else:
                        #retVal = map_Screen.handle_Event(event)
                        #if retVal == 'done':
                        #player_Done = True

                self.render.draw_Screen(map_Screen.render(self.current_Map))
                        
    def fight_Battle(self):
        """Main game loop."""
        no_Winner = True
        current_Player = self.player_1
        
        map_Screen = ctc_screen.map_Screen(self, current_Player)
        self.render.draw_Screen(map_Screen.render(self.current_Map))
        
        while no_Winner:
            map_Screen.player = current_Player
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                else:
                    retVal = map_Screen.handle_Event(event)
                    if retVal == 'done':
                        if current_Player == self.player_1:
                            current_Player = self.player_2
                        else:
                            current_Player = self.player_1
                        
                        self.player_1.reset_Units()
                        self.player_2.reset_Units()
                        map_Screen.tile_Selected = None
                        map_Screen.unit_Selected = None
                    status = self.check_Game_Status()
                    if status == False:
                        continue
                    else:
                        no_Winner = False
                    
            self.render.draw_Screen(map_Screen.render(self.current_Map))

        return status
    def check_Game_Status(self):
        """Checks to see if a win condition has been met."""

        p1_Castle = self.player_1.castle_Location
        p2_Castle = self.player_2.castle_Location

        p1_Units = self.player_1.ALL_UNITS
        p2_Units = self.player_2.ALL_UNITS

        #check if either player's army has been annihilated

        if len(p1_Units) == 0:
            return 2
        elif len(p2_Units) == 0:
            return 1

        #check if either player has a unit on the enemy castle

        for unit in p1_Units:
            if unit.location == p2_Castle:
                return 1

        for unit in p2_Units:
            if unit.location == p1_Castle:
                return 2

        return False        
    def victory(self, winner):
        """Displays a victory screen."""
        
        if winner == 1:
            winner = self.player_1
            loser = self.player_2
        else:
            winner = self.player_2
            loser = self.player_1

        victory_Screen = ctc_screen.victory_Screen(winner, loser)
        self.render.draw_Screen(victory_Screen.render())

        ready = False

        while not ready:
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                else:
                    retVal = victory_Screen.handle_Event(event)
                    if retVal == "done":
                        ready = True                    

            self.render.draw_Screen(victory_Screen.render())
        
        

        pygame.quit()
        sys.exit()
        
    def handle_Event(self, event):
        """Contains control for all higher-level events."""
        #might not use this after all
        pass
        
            
